SystemOrganization addCategory: #TusLibros!

TestCase subclass: #CreditCardTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CreditCardTest methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 18:01'!
testCreatingACreditCardWithDataOKShouldReturnTheCard

	| creditCard expectedNumber expectedOwner expectedExpirationDate |
	
	expectedNumber := '1234'.
	expectedOwner := 'Homero Simpson'.
	expectedExpirationDate := Date tomorrow.
	
	creditCard := CreditCard newWithNumber: expectedNumber andOwner: expectedOwner andExpirationDate: expectedExpirationDate.
	
	self assert: creditCard number equals: expectedNumber.
	self assert: creditCard owner equals: expectedOwner.
	self assert: creditCard expirationDate equals: expectedExpirationDate.! !

!CreditCardTest methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 18:01'!
testCreatingACreditCardWithEmptyNumberShouldRaiseAnError

	 self should: [CreditCard newWithNumber: '' andOwner: 'Homero Simpson' andExpirationDate: Date tomorrow] 
		raise: Error 
		withExceptionDo: [ :anError |  self assert: anError messageText = 'Credit card number cannot be empty.'] ! !

!CreditCardTest methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 18:01'!
testCreatingACreditCardWithEmptyOwnerNameShouldRaiseAnError

	 self should: [CreditCard newWithNumber: '1234' andOwner: '' andExpirationDate: Date tomorrow] 
		raise: Error 
		withExceptionDo: [ :anError |  self assert: anError messageText = 'Credit card owner cannot be empty.'] ! !

!CreditCardTest methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 18:05'!
testCreatingACreditCardWithOwnerNameLengthGreaterThan30ShouldRaiseAnError

	 self should: [CreditCard newWithNumber: '1234' andOwner: '123456789_123456789_123456789_1' andExpirationDate: Date tomorrow] 
		raise: Error 
		withExceptionDo: [ :anError |  self assert: anError messageText = 'Credit card owner cannot be greater than 30 characters long.'] ! !

TestCase subclass: #MerchantProcessorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!MerchantProcessorTest methodsFor: 'Merchant Processor Tests' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 21:22'!
creditCardExpiredResult

	^ CreditCardTransactionResult newWithCode: '1' andMessage: 'EXPIRED_CARD'.! !

!MerchantProcessorTest methodsFor: 'Merchant Processor Tests' stamp: 'EmilianoEzequielNicols 6/6/2013 20:23'!
okResult

	^' ok'.! !

!MerchantProcessorTest methodsFor: 'Merchant Processor Tests' stamp: 'EmilianoEzequielNicols 6/6/2013 20:19'!
testDebit0AmountShouldReturnError

	| merchantProcessor creditCard responseFromMerchantProcessor|
	
	merchantProcessor := PluggableMerchantProcessor  newWith: [ :anAmount | ^'AMOUNT_MUST_BE_GREATER_THAN_ZERO'].
	creditCard := CreditCard newWithNumber: '1234' andOwner: 'Homero Simpson' andExpirationDate: Date tomorrow.
	
	responseFromMerchantProcessor := merchantProcessor debit: 0 fromCard: creditCard.
	
	self assert: responseFromMerchantProcessor equals: self wrongAmountResult.! !

!MerchantProcessorTest methodsFor: 'Merchant Processor Tests' stamp: 'EmilianoEzequielNicols 6/6/2013 20:21'!
testDebitNegativeAmountShouldReturnError

	| merchantProcessor creditCard responseFromMerchantProcessor|
	
	merchantProcessor := PluggableMerchantProcessor  newWith: [ :anAmount | ^'AMOUNT_MUST_BE_GREATER_THAN_ZERO'].
	creditCard := CreditCard newWithNumber: '1234' andOwner: 'Homero Simpson' andExpirationDate: Date tomorrow.
	
	responseFromMerchantProcessor := merchantProcessor debit: -100.50 fromCard: creditCard.
	
	self assert: responseFromMerchantProcessor equals: self wrongAmountResult.! !

!MerchantProcessorTest methodsFor: 'Merchant Processor Tests' stamp: 'EmilianoEzequielNicols 6/6/2013 20:50'!
testDebitWithDataOKShouldReturnOK

	| merchantProcessor creditCard responseFromMerchantProcessor|
	
	merchantProcessor := PluggableMerchantProcessor  newWith: [ :anAmount | ^'ERROR'].
	creditCard := CreditCard newWithNumber: '1234' andOwner: 'Homero Simpson' andExpirationDate: Date tomorrow.
	
	responseFromMerchantProcessor := merchantProcessor  debit: 100.50 fromCard: creditCard.
	
	self assert: responseFromMerchantProcessor equals: self okResult .! !

!MerchantProcessorTest methodsFor: 'Merchant Processor Tests' stamp: 'EmilianoEzequielNicols 6/6/2013 20:48'!
testDebitWithExpiredCardShouldReturnError

	| merchantProcessor creditCard responseFromMerchantProcessor|
	
	merchantProcessor := PluggableMerchantProcessor  newWith: [ :anAmount |^'EXPIRED_CREDIT_CARD'].
	creditCard := CreditCard newWithNumber: '5678' andOwner: 'Marge Simpson' andExpirationDate: Date yesterday.
	
	responseFromMerchantProcessor := merchantProcessor debit: 100.50 fromCard: creditCard.
	
	self assert: responseFromMerchantProcessor equals: self creditCardExpiredResult.! !

!MerchantProcessorTest methodsFor: 'Merchant Processor Tests' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 21:22'!
wrongAmountResult

	^ CreditCardTransactionResult newWithCode: '1' andMessage: 'AMOUNT_MUST_BE_GREATER_THAN_ZERO'.! !

TestCase subclass: #TusLibrosTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TusLibrosTest methodsFor: 'cashier' stamp: 'EmilianoMenendezEzequielNicols 6/3/2013 20:24'!
cajeroWith: aResult

	|aMerchantProcessor|
	
	aMerchantProcessor := MerchantProcessorStub newWith: aResult.
	
	^Cashier newWith: aMerchantProcessor ! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 21:46'!
carrito
	
		^ Cart newCartWithCatalog: self catalogo! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'EmilianoMenendezEzequielNicols 5/31/2013 19:00'!
carritoConUnLibro
     | book1 catalog |
	book1 := '3514564564674'.
	catalog := self catalogo.
	
	^ (Cart newCartWithCatalog: catalog) add: book1! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'EmilianoMenendezEzequielNicols 5/31/2013 19:07'!
catalogo
	| book1 book2 catalog book3 |
	
	book1 := '3514564564674'.
	book2 := '6464564564564'.
	book3:= '35145645646748'.	
	
	catalog := Dictionary new.
	catalog at: book1 put: 2.
	catalog at: book2 put: 25.
	catalog at: book3 put: 27.
	
	^ catalog! !

!TusLibrosTest methodsFor: 'interface' stamp: 'EmilianoEzequielNicols 6/7/2013 22:25'!
now
	^ Clock newWithTimeStamp: TimeStamp now! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicolas 6/7/2013 20:16'!
okMerchantProcessor
	^ PluggableMerchantProcessor newWith: [ 'OK' ]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicols 6/5/2013 21:23'!
securityGuard
	
	^SecurityGuard newWith: self userRegistry.  ! !

!TusLibrosTest methodsFor: 'Credit Card Tests' stamp: 'EmilianoMenendezEzequielNicolas 6/7/2013 20:00'!
tarjetaNoVencida
	
	^ CreditCard newWithNumber: '32442342423' andOwner: 'pepe' andExpirationDate: (Date today + 60)! !

!TusLibrosTest methodsFor: 'Credit Card Tests' stamp: 'EmilianoMenendezEzequielNicols 6/3/2013 20:54'!
tarjetaVencida
	
	^CreditCard newWithNumber: '32442342423' andOwner: 'pepe' andExpirationDate: Date yesterday! !

!TusLibrosTest methodsFor: 'interface' stamp: 'EmilianoEzequielNicols 6/7/2013 23:13'!
tesListAnExpiredCartShouldReturnAnError

	| interface cartId clock |
	
	clock := self now.
	interface := TusLibrosInterface newWithSecurityGuard: self securityGuard andClock: clock.
	
	cartId := interface createCartFor: 'pepe' usingPassword: '1234'.
	
	clock plusMinutes: 31.
	
	self	should: [ interface listCart: cartId ]
		raise: Error
		withExceptionDo: [ :anError | anError messageText: 'Session has expired' ]
	
	
	! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'EmilianoMenendezEzequielNicols 5/27/2013 20:44'!
test01AllCartsAreCreatedEmpty
	"Todo carrito se crea vaco"

	self assert: self carrito isEmpty.! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'EmilianoMenendezEzequielNicols 5/31/2013 19:01'!
test02AfterAddingTheFirstBookCartShouldNotBeEmpty
	"Luego de agregarle libros, el carrito no est vaco"

	self deny: (self carrito add: '6464564564564' withQuantity: 1) isEmpty.! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'EmilianoMenendezEzequielNicols 5/31/2013 19:05'!
test03AfterAddingMoreBooksCartShouldStillNotBeEmpty
	"Luego de agregarle libros, el carrito no est vaco"

	| aBook aCartWithBooks|
	
	aBook:= '3514564564674'.
	
	aCartWithBooks:=  self carrito add: aBook withQuantity: 1.
	
	self deny: (aCartWithBooks add: aBook withQuantity: 1) isEmpty.
	! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 19:54'!
test03BooksAddedShouldAppearWithTheirQuantities
	
	| aBook aCartWithBooks|
	
	aBook:= '35145645646748'.
	
	aCartWithBooks:=  self carrito add: aBook withQuantity: 1.
	aCartWithBooks add: aBook withQuantity: 1.
		
	self assert: (aCartWithBooks hasItem: aBook  withQuantity: 2)! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 19:54'!
test03CartShouldOnlyHaveTheBooksThatWereAdded
	
	| book1 book2 aCartWithBooks|
	
	book1:= '35145645646748'.
	book2:= '6464564564564'.
	
	aCartWithBooks:=  self carrito add: book1 withQuantity: 1.
	aCartWithBooks add: book2 withQuantity: 1.
	
	self assert: (aCartWithBooks hasItem: book1 withQuantity: 1).
	self assert: (aCartWithBooks hasItem: book2 withQuantity: 1).	
	self assert: (aCartWithBooks size = 2)
! !

!TusLibrosTest methodsFor: 'interface' stamp: 'EmilianoEzequielNicols 6/7/2013 22:20'!
testAddToCartNotFoundShouldReturnAnError

	
	| interface cartId clock |
	
	clock := Clock newWithTimeStamp: (TimeStamp now).
	interface := TusLibrosInterface newWithSecurityGuard: (self securityGuard) andClock: clock.
	cartId := interface createCartFor: 'pepe' usingPassword: '1234'.
	
	self should: [interface addToCart: 0 aBookISBN: '3514564564673'  withQuantity: 1]
	raise: Error
	withExceptionDo: [:anError | self assert: anError messageText equals: 'CartId Not Found' ]
	
	
	
	! !

!TusLibrosTest methodsFor: 'interface' stamp: 'EmilianoEzequielNicols 6/7/2013 22:21'!
testAddingAnISBNtoAnActiveCartShouldUpdateTheCart

	| interface cartId cart clock |
		
	clock := Clock newWithTimeStamp: (TimeStamp now).
	interface := TusLibrosInterface newWithSecurityGuard: (self securityGuard) andClock: clock.
	cartId := interface createCartFor: 'pepe' usingPassword: '1234'.
	
	cart := (interface getCartWithId: cartId).
	
	self assert: cart isEmpty.
	
	interface addToCart: cartId aBookISBN: '3514564564674' withQuantity: 1.
	
	self assert: ((cart getQuantityOf: '3514564564674' )= 1)
	
	
	
	! !

!TusLibrosTest methodsFor: 'interface' stamp: 'EmilianoEzequielNicols 6/7/2013 23:27'!
testAddingAnISBNtoAnExpiredCartShouldReturnAnError
	| interface cartId clock |
	
	clock := self now.
	interface := TusLibrosInterface newWithSecurityGuard: self securityGuard andClock: clock.
	
	cartId := interface createCartFor: 'pepe' usingPassword: '1234'.
	
	clock plusMinutes: 31.
	
	self	should: [ interface addToCart: cartId aBookISBN: '1234556' withQuantity: 1 ]
		raise: Error
		withExceptionDo: [ :anError | anError messageText: 'Session has Expired' ]! !

!TusLibrosTest methodsFor: 'cashier' stamp: 'EmilianoMenendezEzequielNicolas 6/7/2013 20:27'!
testCheckoutOfCartWithSingleElementShouldReturnSinglePrice

	|cart aCashier expectedTotal |
	
	
	aCashier:= Cashier newWith: self okMerchantProcessor.
	cart := self carritoConUnLibro.
	expectedTotal :=  aCashier calculateTotal: cart.
	
	self assert: ((aCashier checkOut: cart payingWith: self tarjetaNoVencida) total = expectedTotal) ! !

!TusLibrosTest methodsFor: 'cashier' stamp: 'EmilianoMenendezEzequielNicolas 6/7/2013 20:20'!
testCheckoutOfEmptyCartShouldNotAddNewSale
	| aCart aCashier newSale purchaseTotal |
	aCart := self carritoConUnLibro.
	aCashier := Cashier newCashier.
	purchaseTotal := aCashier calculateTotal: aCart.
	newSale := Ticket newFrom: aCart withTotal: purchaseTotal andTransactionResult: aResult.
	self deny: (aCashier libroDeVentas includes: newSale)! !

!TusLibrosTest methodsFor: 'cashier' stamp: 'EmilianoMenendezEzequielNicolas 6/7/2013 20:26'!
testCheckoutOfEmptyCartShouldRaiseAnError

	|emptyCart aCashier|

	emptyCart := Cart newCartWithCatalog: Bag new.
      aCashier := Cashier new. 

 	self should: [aCashier checkOut: emptyCart payingWith: self tarjetaNoVencida ] 
		raise: Error 
		withExceptionDo: [ :anError |  self assert: anError messageText = Cashier cantCheckoutEmptyCartMessage]  ! !

!TusLibrosTest methodsFor: 'cashier' stamp: 'EmilianoMenendezEzequielNicolas 6/7/2013 19:30'!
testCheckoutWithExpiredCardDoesNotCallTheMerchantProcessor

	|cashier merchantProcessor|
	
	merchantProcessor := PluggableMerchantProcessor new initializeWith: [ self fail: 'MerchantProcessor should not had called using expired card.' ].
	cashier := Cashier newWith: merchantProcessor.
	
	self should: [cashier checkOut: self carritoConUnLibro payingWith: self tarjetaVencida]
	raise: Error
	withExceptionDo: [:anError | self assert: anError messageText equals: 'EXPIRED_CARD' ]! !

!TusLibrosTest methodsFor: 'cashier' stamp: 'EmilianoMenendezEzequielNicolas 6/7/2013 19:34'!
testCheckoutWithStolenCardShouldRejectTransaction

	|cashier merchantProcessor |
	
	merchantProcessor := PluggableMerchantProcessor new initializeWith: [ Error signal: 'Card was stolen.'].
	cashier := Cashier newWith: merchantProcessor.
	
	self should: [cashier checkOut: self carritoConUnLibro payingWith: self tarjetaNoVencida ]
	raise: Error
	withExceptionDo: [:anError | self assert: anError messageText equals: 'Card was stolen.' ]! !

!TusLibrosTest methodsFor: 'interface' stamp: 'EmilianoEzequielNicols 6/7/2013 23:03'!
testCreateCartWithValidCredentialsShouldReturnANewCart
	
	| interface cartId cart clock session |
	
	clock := Clock newWithTimeStamp: TimeStamp now.
	interface := TusLibrosInterface newWithSecurityGuard: self securityGuard andClock: clock.
	
	cartId := interface createCartFor: 'pepe' usingPassword: '1234'.
	
	session := interface getSessionWithId: cartId.
	self deny: session isExpired.
	
	cart := interface getCartWithId: cartId.
	self assert: cart isEmpty.
	
	
! !

!TusLibrosTest methodsFor: 'Credit Card Tests' stamp: 'EmilianoMenendezEzequielNicols 6/2/2013 17:23'!
testCreatingACreditCardWithDataOKShouldReturnTheCard

	| creditCard expectedNumber expectedOwner expectedExpirationDate |
	
	expectedNumber := '1234'.
	expectedOwner := 'Homero Simpson'.
	expectedExpirationDate := Date tomorrow.
	
	creditCard := CreditCard newWithNumber: expectedNumber andOwner: expectedOwner andExpirationDate: expectedExpirationDate.
	
	self assert: creditCard number equals: expectedNumber.
	self assert: creditCard owner equals: expectedOwner.
	self assert: creditCard expirationDate equals: expectedExpirationDate.! !

!TusLibrosTest methodsFor: 'Credit Card Tests' stamp: 'EmilianoMenendezEzequielNicols 6/2/2013 17:23'!
testCreatingACreditCardWithEmptyNumberShouldRaiseAnError

	 self should: [CreditCard newWithNumber: '' andOwner: 'Homero Simpson' andExpirationDate: Date tomorrow] 
		raise: Error 
		withExceptionDo: [ :anError |  self assert: anError messageText = 'Credit card number cannot be empty.'] ! !

!TusLibrosTest methodsFor: 'Credit Card Tests' stamp: 'EmilianoMenendezEzequielNicols 6/2/2013 17:23'!
testCreatingACreditCardWithEmptyOwnerNameShouldRaiseAnError

	 self should: [CreditCard newWithNumber: '1234' andOwner: '' andExpirationDate: Date tomorrow] 
		raise: Error 
		withExceptionDo: [ :anError |  self assert: anError messageText = 'Credit card owner cannot be empty.'] ! !

!TusLibrosTest methodsFor: 'Credit Card Tests' stamp: 'EmilianoMenendezEzequielNicols 6/2/2013 17:23'!
testCreatingACreditCardWithOwnerNameLengthGreaterThan30ShouldRaiseAnError

	 self should: [CreditCard newWithNumber: '1234' andOwner: '123456789_123456789_123456789_1' andExpirationDate: Date tomorrow] 
		raise: Error 
		withExceptionDo: [ :anError |  self assert: anError messageText = 'Credit card owner cannot be greater than 30 characters long.'] ! !

!TusLibrosTest methodsFor: 'interface' stamp: 'EmilianoEzequielNicols 6/7/2013 23:15'!
testListAnExpiredCartShouldReturnAnError

	| interface cartId clock |
	
	clock := self now.
	interface := TusLibrosInterface newWithSecurityGuard: self securityGuard andClock: clock.
	
	cartId := interface createCartFor: 'pepe' usingPassword: '1234'.
	
	clock plusMinutes: 31.
	
	self	should: [ interface listCart: cartId ]
		raise: Error
		withExceptionDo: [ :anError | anError messageText: 'Session has expired' ]
	
	
	! !

!TusLibrosTest methodsFor: 'interface' stamp: 'EmilianoEzequielNicols 6/7/2013 23:04'!
testListCartNotFoundShouldReturnAnError

	| interface cartId clock |
	
	clock := Clock newWithTimeStamp: (TimeStamp now).
	interface := TusLibrosInterface newWithSecurityGuard: (self securityGuard) andClock: clock.
	cartId := interface createCartFor: 'pepe' usingPassword: '1234'.
	
	self should: [interface listCart: 0]
	raise: Error
	withExceptionDo: [:anError | self assert: anError messageText equals: 'CartId Not Found' ]
	
	
	
	! !

!TusLibrosTest methodsFor: 'security' stamp: 'EmilianoMenendezEzequielNicols 6/5/2013 21:14'!
testSuccessfulUserAuthenticationShouldReturnAToken
	| token |

	token := self securityGuard validate: 'pepe' with: '1234'.

	self assert: token > 0.! !

!TusLibrosTest methodsFor: 'security' stamp: 'EmilianoMenendezEzequielNicols 6/5/2013 21:19'!
testTryingToAuthenticateInvalidUserNameShouldRaiseError

	self should: [self securityGuard validate: 'not_a_user' with: '1234'] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = 'Invalid User or Password.' ]
! !

!TusLibrosTest methodsFor: 'security' stamp: 'EmilianoMenendezEzequielNicols 6/5/2013 21:20'!
testTryingToAuthenticateUserWithInvalidPasswordShouldRaiseError

	self should: [self securityGuard validate: 'pepe' with: 'wrong_password'] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = 'Invalid User or Password.' ]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicols 6/5/2013 21:23'!
userRegistry
	
	|userRegistry |
	
	userRegistry := Dictionary  new.
	userRegistry at:'pepe' put:'1234'.
	^userRegistry 
	! !

Object subclass: #Cart
	instanceVariableNames: 'items catalogo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cart class methodsFor: 'instance-creation' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 20:13'!
newCartWithCatalog: aCatalog
	^ self new emptyCartWithCatalog: aCatalog! !

!Cart methodsFor: 'adding' stamp: 'EmilianoMenendezEzequielNicols 5/31/2013 19:03'!
add: anISBN withQuantity: aQuantity 
	
	
	(catalogo includesKey: anISBN ) ifFalse:[^ Error signal: 'El libro no pertenece al catlogo' ].
	items add: anISBN withOccurrences: aQuantity ! !

!Cart methodsFor: 'testing' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 20:01'!
catalog

	^catalogo! !

!Cart methodsFor: 'Initializing' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 19:58'!
emptyCartWithCatalog: aCatalog
	
	items := Bag new.
	catalogo := aCatalog.! !

!Cart methodsFor: 'testing' stamp: 'EmilianoEzequielNicols 6/7/2013 21:24'!
getQuantityOf: anItem 

	^items occurrencesOf: anItem  ! !

!Cart methodsFor: 'testing' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 19:54'!
hasItem: anISBN withQuantity: aQuantity

	^(items occurrencesOf: anISBN) = aQuantity  ! !

!Cart methodsFor: 'testing' stamp: 'EmilianoMenendezEzequielNicols 5/27/2013 21:46'!
isEmpty

	^items isEmpty! !

!Cart methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielNicols 6/1/2013 15:22'!
itemPrice: anItem 
	
	^catalogo at: anItem! !

!Cart methodsFor: 'accessing' stamp: 'EmilianoEzequielNicols 6/1/2013 16:07'!
itemQuantity: anItem  
	
	^items occurrencesOf: anItem! !

!Cart methodsFor: 'testing' stamp: 'EmilianoEzequielNicols 6/1/2013 16:25'!
items

	^items ! !

!Cart methodsFor: 'testing' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 19:32'!
size

	^items size! !

Object subclass: #Cashier
	instanceVariableNames: 'libroDeVentas merchantProcessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cashier class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 20:42'!
cantCheckoutEmptyCartMessage
	^ 'Cant checkout an empty cart'! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicols 6/2/2013 17:52'!
newCashier
	
	
	^ self new initialize.  ! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicols 6/3/2013 20:20'!
newWith: aMerchantProcessor
	
	
	^ self new initializeWith: aMerchantProcessor.  ! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicols 6/2/2013 17:38'!
calculateTotal: aCart 
	
	|cartItems temporaryTotal|
	
	cartItems := aCart items.
	temporaryTotal := 0.
	
	cartItems do: [ :anItem | temporaryTotal := temporaryTotal + (( aCart itemQuantity: anItem) * (aCart itemPrice: anItem))].
	
	^temporaryTotal.! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicolas 6/7/2013 20:12'!
checkOut: aCart payingWith: aCreditCard
	| ticket aTotal transactionResult |
	
	aCart isEmpty
		ifTrue: [ ^ Error signal: self class cantCheckoutEmptyCartMessage ].
	(aCreditCard isExpired: Date today)
		ifTrue: [ ^ Error signal: 'EXPIRED_CARD' ].

	aTotal := self calculateTotal: aCart.
	transactionResult := merchantProcessor debit: aTotal fromCard: aCreditCard.
	ticket := Ticket newFrom: aCart withTotal: aTotal andTransactionResult: transactionResult.
	
	(transactionResult  = 'OK') ifTrue: [libroDeVentas add: ticket].

	^ ticket! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicols 6/2/2013 17:53'!
initialize

	libroDeVentas := Bag new.! !

!Cashier methodsFor: 'initializing' stamp: 'EmilianoMenendezEzequielNicols 6/3/2013 20:39'!
initializeWith: aMerchantProcessorStub 
	
	libroDeVentas := Bag new.
	merchantProcessor := aMerchantProcessorStub! !

!Cashier methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielNicols 6/3/2013 19:20'!
libroDeVentas

	^ libroDeVentas! !

Object subclass: #Clock
	instanceVariableNames: 'timestamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Clock class methodsFor: 'as yet unclassified' stamp: 'EmilianoEzequielNicols 6/7/2013 19:27'!
newWithTimeStamp: aTimestamp 

	^self new initializeWithTimestamp: aTimestamp.! !

!Clock methodsFor: 'initializing' stamp: 'EmilianoEzequielNicols 6/7/2013 19:37'!
initializeWithTimestamp: aTimestamp 
	
	timestamp := aTimestamp ! !

!Clock methodsFor: 'accesing' stamp: 'EmilianoEzequielNicols 6/7/2013 22:14'!
plusMinutes: minutes 

	timestamp plusSeconds: minutes * 60. ! !

!Clock methodsFor: 'accesing' stamp: 'EmilianoEzequielNicols 6/7/2013 20:12'!
timeStamp

	^timestamp ! !

!Clock methodsFor: 'accesing' stamp: 'EmilianoEzequielNicols 6/7/2013 20:16'!
timeStamp: aNewTimestamp 

	timestamp := aNewTimestamp ! !

Object subclass: #CreditCard
	instanceVariableNames: 'number owner expirationDate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CreditCard class methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 17:37'!
newWithNumber: aNumber andOwner: anOwner andExpirationDate: anExpirationDate
	^ self new initializeWithNumber: aNumber andOwner: anOwner andExpirationDate: anExpirationDate.! !

!CreditCard methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 17:39'!
expirationDate
	^ expirationDate! !

!CreditCard methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 18:45'!
initializeWithNumber: aNumber andOwner: anOwner andExpirationDate: anExpirationDate

	(aNumber trim size = 0) ifTrue: [ ^ Error signal: 'Credit card number cannot be empty.' ].
	(anOwner trim size = 0) ifTrue: [ ^ Error signal: 'Credit card owner cannot be empty.' ].
	(anOwner trim size > 30) ifTrue: [ ^ Error signal: 'Credit card owner cannot be greater than 30 characters long.' ].
	
	number := aNumber trim.
	owner := anOwner.
	expirationDate := anExpirationDate.! !

!CreditCard methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielNicolas 6/7/2013 19:51'!
isExpired: aDate
	^ expirationDate < aDate! !

!CreditCard methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 17:38'!
number
	^ number! !

!CreditCard methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 17:39'!
owner
	^ owner! !

Object subclass: #PluggableMerchantProcessor
	instanceVariableNames: 'result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!PluggableMerchantProcessor class methodsFor: 'as yet unclassified' stamp: 'EmilianoEzequielNicols 6/6/2013 20:15'!
newWith: aBlock 
	
	^self new initializeWith: aBlock! !

!PluggableMerchantProcessor methodsFor: 'operation' stamp: 'EmilianoMenendezEzequielNicolas 6/7/2013 20:01'!
debit: anAmount fromCard: aCreditCard 
	
	result value! !

!PluggableMerchantProcessor methodsFor: 'initilization' stamp: 'EmilianoEzequielNicols 6/6/2013 20:27'!
initializeWith: aResult

	result := aResult! !

Object subclass: #SecurityGuard
	instanceVariableNames: 'userRegistry'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!SecurityGuard class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicols 6/5/2013 20:09'!
newWith: aRegistry

	^self new initializeWith: aRegistry! !

!SecurityGuard methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicols 6/5/2013 20:10'!
initializeWith: aRegistry

	userRegistry:= aRegistry ! !

!SecurityGuard methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicols 6/5/2013 21:16'!
validate: aUser with: aPassword

	((userRegistry includesKey: aUser) and: [(userRegistry at: aUser ) = aPassword ]) ifTrue: [^ Random new nextInt: 10000000].
	^ Error signal: 'Invalid User or Password.'! !

Object subclass: #Session
	instanceVariableNames: 'creationTime lastAccessTime cart clock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Session class methodsFor: 'as yet unclassified' stamp: 'EmilianoEzequielNicols 6/7/2013 21:36'!
newWithCreationTime: aTime andLastAccessTime: anotherTime andCart:aCart andTestingClock: aClock
	
	^self new initializeWithCreationTime: aTime andLastAccessTime: anotherTime andCart: aCart andTestingClock: aClock! !

!Session methodsFor: 'expiration' stamp: 'EmilianoEzequielNicols 6/7/2013 21:51'!
KeepActive
	
	|newTimestamp|
	
	newTimestamp := (TimeStamp now) time.
	clock timeStamp: newTimestamp.! !

!Session methodsFor: 'accesing' stamp: 'EmilianoEzequielNicols 6/7/2013 00:14'!
cart
	
	^cart! !

!Session methodsFor: 'initializing' stamp: 'EmilianoEzequielNicols 6/7/2013 21:51'!
initializeWithCreationTime: aTime andLastAccessTime: anotherTime andCart: aCart andTestingClock: aClock 
	
	creationTime := aTime.
	lastAccessTime := anotherTime.
	cart := aCart.
	clock := aClock! !

!Session methodsFor: 'expiration' stamp: 'EmilianoEzequielNicols 6/7/2013 22:43'!
isExpired
	
	|secondsIn30Minutes|
	
	secondsIn30Minutes := 1800.
	
	^(clock timeStamp time) >  (lastAccessTime plusSeconds: secondsIn30Minutes )  
	! !

!Session methodsFor: 'operating' stamp: 'EmilianoEzequielNicols 6/7/2013 23:34'!
setLastAccessTime: aTimeStamp 
	lastAccessTime := aTimeStamp ! !

Object subclass: #Ticket
	instanceVariableNames: 'items total transactionResult'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Ticket class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicolas 6/7/2013 20:20'!
newFrom: aCart withTotal: anAmount andTransactionResult: anObject
	^ Ticket new initializeWith: aCart andTotal: anAmount andTransactionResult: aResult! !

!Ticket methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicols 6/4/2013 21:11'!
addCartItems: aCart 
	
	|ticketItems|
	
	ticketItems := OrderedCollection new: aCart items size.
	
! !

!Ticket methodsFor: 'initializing' stamp: 'EmilianoMenendezEzequielNicols 6/4/2013 19:20'!
initializeWith: aCart andTotal: anAmount andTransactionResult: anObject
	
	
	items :=  self addCartItems: aCart.
	total := anAmount.
	! !

!Ticket methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielNicols 6/1/2013 14:36'!
total
	^ total! !

Object subclass: #TusLibrosInterface
	instanceVariableNames: 'securityGuard sessionsRegistry clock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TusLibrosInterface class methodsFor: 'creation' stamp: 'EmilianoEzequielNicols 6/7/2013 22:17'!
newWithSecurityGuard: aSecurityGuard andClock: aClock
	
	
	^self new initializeWithSecurityGuard: aSecurityGuard andClock: aClock ! !

!TusLibrosInterface methodsFor: 'operation' stamp: 'EmilianoEzequielNicols 6/7/2013 23:32'!
addToCart: aCartId aBookISBN: anISBN withQuantity:aQuantity

	|cart session newLastAccessTime|
	
	((sessionsRegistry includesKey: aCartId)not) ifTrue: [^Error signal: 'CartId Not Found']. 
	
	session := self getSessionWithId: aCartId.
	(session isExpired) ifTrue: [^Error signal: 'Session has Expired'].
	 
	cart := self getCartWithId: aCartId.
	cart add: anISBN  withQuantity: aQuantity.
	
	newLastAccessTime := (TimeStamp now).
	session setLastAccessTime: newLastAccessTime
	
	 
	
! !

!TusLibrosInterface methodsFor: 'operation' stamp: 'EmilianoEzequielNicols 6/7/2013 21:52'!
createCartFor: aUsername usingPassword: aPassword  
	
	| cartId  catalog cart newSession timestamp clock |
	
	cartId  := securityGuard validate: aUsername  with: aPassword.
	catalog := (TusLibrosTest new) catalogo.
	cart := Cart newCartWithCatalog: catalog. 
	timestamp := (TimeStamp now).
	
	clock :=  Clock newWithTimeStamp: timestamp.
	newSession := Session newWithCreationTime: timestamp  andLastAccessTime:  timestamp andCart: cart  andTestingClock: clock.
	
	sessionsRegistry at: cartId put: newSession.
	
	^cartId ! !

!TusLibrosInterface methodsFor: 'accesing' stamp: 'EmilianoEzequielNicols 6/7/2013 20:41'!
getCartWithId: aCartId 

	^(sessionsRegistry at: aCartId ) cart! !

!TusLibrosInterface methodsFor: 'accesing' stamp: 'EmilianoEzequielNicols 6/7/2013 21:47'!
getSessionWithId: aCartId 

	^(sessionsRegistry at: aCartId )! !

!TusLibrosInterface methodsFor: 'initializing' stamp: 'EmilianoEzequielNicols 6/7/2013 22:19'!
initializeWithSecurityGuard: aSecurityGuard andClock: aClock 
	
	securityGuard := aSecurityGuard. 
	sessionsRegistry  := Dictionary new.
	clock := aClock
	! !

!TusLibrosInterface methodsFor: 'operation' stamp: 'EmilianoEzequielNicols 6/7/2013 23:38'!
listCart: aCartId 

	|cart session |
	
	((sessionsRegistry includesKey: aCartId)not) ifTrue: [^Error signal: 'CartId Not Found']. 
	
	session := self getSessionWithId: aCartId.
	((session isExpired) isExpired ) ifTrue: [ ^Error signal: 'Session has Expired' ]. 
	
	cart := self getCartWithId: aCartId.
	session setLastAccessTime: (TimeStamp now).
	
	^cart items 
	
	 
	
! !
